#!/usr/local/bin/bash
#
# Dirk Dettmering
#
# Version 0.13 
# 23.01.2019
#

USAGE="Syntax: $0 domain.tld altdomain.tld subdomain.domain.tld
       Erster Parameter ist die primaer geschuetzte Domain und
       dann kommen die alternativen Domainnamen"

if [ "$#" == "0" ]; then
    echo "$USAGE"
    exit 1
fi

# Variablen für Einstellunge:
KEYDIR="/usr/local/etc/letsencrypt"
#KEYDIR="/usr/local/etc/letstest"
ACMETINY="/usr/local/bin/acme_tiny"
# Verzeichnis für die ACME-Challenge (Port 80!!!, nicht 443!!!)
ACMEDIR="/usr/local/www/challenges/"
# ACMEDIR="/usr/local/www/challenges/.well-known/acme-challenge/"
# Standard-Unterverzeichnis für die ACME-Challenge (Port 80!!!, nicht 443!!!)
# ACMESUBDIR=".well-known/acme-challenge/"

# Benutzer Variablen für Einstellungen
ACCOUNTKEY=$KEYDIR/account.key
DOMAIN="$1"
shift
for num
    do
	ALTDOMAINS=$ALTDOMAINS" "$num
done

BACKUPDIR=$KEYDIR/backup/$DOMAIN/`date +%Y-%m-%d`
LOG=$KEYDIR/logs/$DOMAIN.log
LOGtmp=$KEYDIR/logs/$DOMAIN_tmp.log

# SSL-Konfiguration:
SSLCNF=$KEYDIR/$DOMAIN/$DOMAIN.cnf
COUNTRY="DE"
STATEORPROVINCENAME="Hessen"
CITY="Musterstadt"
ORGANIZATIONNAME="Musterfirma"
ORGANIZATIONALUNITNAME="IT-Abteilung"
EMAIL="cert-admin@domain.tld"
#
# Servicerestarts:
HTTPRESTART="/usr/local/sbin/apachectl graceful"
IMAPRESTART="service dovecot restart"
SMTPRESTART="service postfix restart"
CHOWN="root:wheel"
# Weitere Vaiablen (Diese müssen im Normallfall nicht geändert werden)
CHMOD="644"
# Wird auf false gesetzt, wenn was schief läuft
CRTOK=true
# Ende der Benutzer-Variablen.

# Ab hier beginnt das Script.

# Es wird geprüft, ob acme-tiny vorhanden ist
if [ ! -f $ACMETINY ]; then
    echo "ACME_TINY nicht gefunden"
    exit 2
fi

# Datum und Variablen werden ins LOG geschrieben
echo "--- Start ---" >$LOG
date >>$LOG
echo "Domain: $DOMAIN" >>$LOG
echo "Accountkey: $ACCOUNTKEY" >>$LOG
echo "------------------------" >>$LOG

# Ordner für die Domain erstellen, falls nocht nicht vorhande:
if [ ! -d $KEYDIR/$DOMAIN ]; then
    mkdir -p $KEYDIR/$DOMAIN
fi

# Backup-Ordner fuer aktuelle Zertifikate wird erstellt
if [ -s $KEYDIR/$DOMAIN/$DOMAIN.pem ] && [ ! -d $BACKUPDIR ]; then
    mkdir -p $BACKUPDIR
    mv -f $KEYDIR/$DOMAIN/lets-encrypt-x3-cross-signed-intermediate.pem $BACKUPDIR > /dev/null 2>&1
    mv -f $KEYDIR/$DOMAIN/$DOMAIN.crt $BACKUPDIR > /dev/null 2>&1
    mv -f $KEYDIR/$DOMAIN/$DOMAIN.csr $BACKUPDIR > /dev/null 2>&1
    mv -f $KEYDIR/$DOMAIN/$DOMAIN.pem $BACKUPDIR > /dev/null 2>&1
    mv -f $KEYDIR/$DOMAIN/$DOMAIN-chain.pem $BACKUPDIR > /dev/null 2>&1
fi

# Es wird geprüft, ob der ACCOUNT-Key existiert.
if [ ! -f $ACCONTKEY ]; then
    echo "Account Key $ACCOUNTKEY nicht vorhanden" >> $LOG 2>&1
    echo "Key kann mit dem folgenden Befehl erstellt werden:" >> $LOG 2>&1
    echo "openssl genrsa 4096 > $ACCOUNTKEY" >> $LOG 2>&1
    exit 3
fi

# prüfen, ob die Domain erreichbar ist:
if /usr/bin/nc -z $DOMAIN 80 > /dev/null 2>&1; then
    echo "Domain erreichbar" >>$LOG 2>&1
else
    echo "Domain nicht erreichbar" >>$LOG 2>&1
    exit 4
fi

# Private Key der Domain erstellen, wenn noch nicht vorhanden:
if [ ! -f $KEYDIR/$DOMAIN/$DOMAIN.key ]; then
    openssl genrsa 4096 > $KEYDIR/$DOMAIN/$DOMAIN.key 2>>$LOG 2>&1
fi

# Config-Datei für CSR erstelen, wenn noch nicht vorhanden:
if [ ! -f $SSLCNF ]; then
    echo "[ req ]
    distinguished_name = req_distinguished_name
    string_mask = nombstr
    # The extensions to add to a certificate request
    req_extensions = v3_req
    # GWDG default options for certificate request
    [ req_distinguished_name ]
    countryName = Country Name (2 letter code)" > $SSLCNF
    echo -n "countryName_default = " >> $SSLCNF
    echo $COUNTRY >> $SSLCNF
    echo "countryName_min = 2" >> $SSLCNF
    echo "countryName_max = 2" >> $SSLCNF
    echo "stateOrProvinceName = State or Province Name (full name)" >> $SSLCNF
    echo -n "stateOrProvinceName_default = " >> $SSLCNF
    echo $STATEORPROVINCENAME >> $SSLCNF
    echo "localityName = Your City" >> $SSLCNF
    echo -n "localityName_default = " >> $SSLCNF
    echo $CITY >> $SSLCNF
    echo "0.organizationName = Organization Name (eg, company)
    0.organizationName_default = "$ORGANIZATIONNAME >> $SSLCNF
    echo "organizationalUnitName = Organizational Unit Name (eg, section)
    organizationalUnitName_default = "$ORGANIZATIONALUNITNAME >> $SSLCNF
    echo "commonName = Common Name (e.g. server FQDN or YOUR name)" >> $SSLCNF
    echo "commonName_max = 64" >> $SSLCNF
    echo -n "commonName_default = " >> $SSLCNF
    echo $DOMAIN >> $SSLCNF
    echo "emailAddress = Email Address" >> $SSLCNF
    echo "emailAddress_max = 64" >> $SSLCNF
    echo -n "emailAddress_default = " >> $SSLCNF
    echo $EMAIL >> $SSLCNF 
    echo "[ v3_req ]" >> $SSLCNF
    echo -n "subjectAltName = DNS:" >> $SSLCNF
    echo -n $DOMAIN >> $SSLCNF
fi

for SUB in $ALTDOMAINS; do
    echo -n ", DNS:" >> $SSLCNF
    echo -n $SUB >> $SSLCNF
done

# CSR wird erstellt
# openssl req -new -sha256 -key /etc/ssl/letsencrypt/domain.key -subj "/C=US/O=Acme/CN=example.com" -reqexts SAN -config <(cat /etc/ssl/openssl.cnf <(printf "[SAN]\nsubjectAltName=DNS:www.example.com,DNS:static.example.com")) -out /etc/ssl/letsencrypt/example.com/domain.csr
openssl req -new -sha256 -key $KEYDIR/$DOMAIN/$DOMAIN.key -batch -config $KEYDIR/$DOMAIN/$DOMAIN.cnf -out $KEYDIR/$DOMAIN/$DOMAIN.csr >> $LOG 2>&1

# CSR von Lets-Encrypt signieren lassen
/usr/local/bin/python2.7 $ACMETINY --account-key $ACCOUNTKEY --csr $KEYDIR/$DOMAIN/$DOMAIN.csr --acme-dir $ACMEDIR > $KEYDIR/$DOMAIN/$DOMAIN.crt
RESULT=$?
if [ ! $RESULT == 0 ];then
    echo "Signierung nicht erfolgreich!" >>$LOG
    echo "Signierung nicht erfolgreich!"
    exit $RESULT
else
    echo "Signierung erfolgreich!" >>$LOG
fi

# Append the Let's Encrypt intermediate cert to your cert
/usr/local/bin/wget -q -O - https://letsencrypt.org/certs/lets-encrypt-x3-cross-signed.pem > $KEYDIR/$DOMAIN/lets-encrypt-x3-cross-signed-intermediate.pem 2>>$LOG 2>&1
cat $KEYDIR/$DOMAIN/$DOMAIN.crt $KEYDIR/$DOMAIN/lets-encrypt-x3-cross-signed-intermediate.pem >> $KEYDIR/$DOMAIN/$DOMAIN-chain.pem

# Rechte
/usr/sbin/chown -R $CHOWN $KEYDIR/$DOMAIN 
/bin/chmod -R $CHMOD $KEYDIR/$DOMAIN/*

# Jetzt kommen die Prüfungen, ob das CRT OK ist.
if [ `grep -c CERTIFICATE $KEYDIR/$DOMAIN/$DOMAIN-chain.pem` = 6 ]; then
    echo "Zertifikat ist OK" >>$LOG
    # PEM-Datei wird erstellt (PEM = KEY + CRT)
    cat $KEYDIR/$DOMAIN/$DOMAIN.key >$KEYDIR/$DOMAIN/$DOMAIN.pem
    cat $KEYDIR/$DOMAIN/$DOMAIN-chain.pem >>$KEYDIR/$DOMAIN/$DOMAIN.pem
else
    CRTOK=false
    echo "Zertifikat ist Fehlerhaft" >>$LOG
    exit 7
fi

# Hier wird anhand des Prüfungsergebnisses weiter entschieden
if $CRTOK; then
    # CRT ist OK

    # Webserver neu Starten!
    # $HTTPRESTARTCMD 2>/dev/null >/dev/null
    $HTTPRESTARTCMD  > /dev/null 2>&1
    # Mailserver neu Starten!
    $IMAPRESTART > /dev/null 2>&1
    #$SMTPRESTART > /dev/null 2>&1
else
    # Fehler beim CRT
    mkdir -p $KEYDIR/failed/$DOMAIN-`date +%Y-%m-%d-%H-%M`
    mv $KEYDIR/$DOMAIN/* $KEYDIR/failed/$DOMAIN_`date +%Y-%m-%d_%H-%M`
    exit 5
fi
